const express = require('express');
const SubCategoriesDBApi = require('../../db/api/sub_categories');
const wrapAsync = require('../helpers').wrapAsync;

const router = express.Router();

/**
 * @swagger
 * /api/sub_categories:
 *   get:
 *     security:
 *       - bearerAuth: []
 *     tags: [SubCategories]
 *     summary: Get all sub_categories
 *     description: Fetch all sub-categories with optional filtering.
 *     parameters:
 *       - in: query
 *         name: name
 *         schema:
 *           type: string
 *         description: Filter by name (case insensitive)
 *     responses:
 *       200:
 *         description: SubCategories list successfully received.
 *         content:
 *           application/json:
 *             schema:
 *               type: array
 *               items:
 *                 $ref: "#/components/schemas/SubCategories"
 *       401:
 *         $ref: "#/components/responses/UnauthorizedError"
 *       404:
 *         description: Data not found
 *       500:
 *         description: Internal server error
 */
router.get(
  '/',
  wrapAsync(async (req, res) => {
    const payload = await SubCategoriesDBApi.findAll(req.query);
    res.status(200).send(payload);
  }),
);

router.get(
  '/:id',
  wrapAsync(async (req, res) => {
    const record = await SubCategoriesDBApi.findBy({ id: req.params.id });
    if (!record) {
      return res.status(404).json({ error: 'Record not found' });
    }
    res.status(200).json(record);
  }),
);

router.post(
  '/',
  wrapAsync(async (req, res) => {
    const { data } = req.body;
    if (!data || !data.name || !data.category_id) {
      return res.status(400).json({ error: 'Invalid data provided' });
    }
    const record = await SubCategoriesDBApi.create(data);
    res.status(201).json(record);
  }),
);

router.put(
  '/:id',
  wrapAsync(async (req, res) => {
    const { data } = req.body;
    if (!data) {
      return res.status(400).json({ error: 'Invalid data provided' });
    }
    const record = await SubCategoriesDBApi.update(req.params.id, data);
    res.status(200).json(record);
  }),
);

router.delete(
  '/:id',
  wrapAsync(async (req, res) => {
    await SubCategoriesDBApi.remove(req.params.id);
    res.status(200).json({ message: 'Record deleted successfully' });
  }),
);

module.exports = router;
