import { Injectable, OnModuleInit } from '@nestjs/common';
import { ConfigService } from '../config/config.service';
import { Sequelize } from 'sequelize';
import * as path from 'path';
import * as fs from 'fs';

@Injectable()
export class DatabaseService implements OnModuleInit {
  public sequelize: Sequelize;
  public db: any = {};

  constructor(private configService: ConfigService) {}

  async onModuleInit() {
    // Log database configuration
    const dbConfig = {
      host: this.configService.getRequired('DB_HOST'),
      port: this.configService.getNumber('DB_PORT') || 3306,
      username: this.configService.getRequired('DB_USERNAME'),
      database: this.configService.getRequired('DB_DATABASE'),
      hasPassword: !!this.configService.getRequired('DB_PASSWORD'),
    };
    
    console.log('Database Configuration:', dbConfig);

    // Initialize Sequelize
    this.sequelize = new Sequelize({
      dialect: 'mysql',
      host: dbConfig.host,
      port: dbConfig.port,
      username: dbConfig.username,
      password: this.configService.getRequired('DB_PASSWORD'),
      database: dbConfig.database,
      logging: false,
      define: {
        timestamps: true,
        paranoid: true,
        underscored: true,
        freezeTableName: true
      }
    });

    // Load models
    const modelsPath = path.join(__dirname, '../db/models');
    
    if (fs.existsSync(modelsPath)) {
      const modelFiles = fs.readdirSync(modelsPath)
        .filter(file => file.indexOf('.') !== 0 && file.slice(-3) === '.ts' && file.slice(-5) !== '.d.ts');
        
      for (const file of modelFiles) {
        const modelModule = await import(path.join(modelsPath, file));
        const model = modelModule.default
          ? modelModule.default(this.sequelize, Sequelize)
          : modelModule(this.sequelize, Sequelize);
        this.db[model.name] = model;
      }

      // Associate models
      Object.keys(this.db).forEach(modelName => {
        if (this.db[modelName].associate) {
          this.db[modelName].associate(this.db);
        }
      });
    }

    // Test connection
    try {
      await this.sequelize.authenticate();
      console.log('Database connection established successfully.');
    } catch (error) {
      console.error('Unable to connect to the database:', error);
      throw error;
    }
  }

  getDb() {
    return this.db;
  }

  getSequelize() {
    return this.sequelize;
  }
}