
const Utils = require("../utils");
const Sequelize = require("sequelize");
const Op = Sequelize.Op;
const getDbInstance = require("../initialize");
const { v4: uuid } = require("uuid");
module.exports = class SamagriDBApi {
  static async create(data, options) {
    const db = await getDbInstance();
    const currentUser = (options && options.currentUser) || { id: null };
    const transaction = (options && options.transaction) || undefined;

    const samagri = await db.samagri.create(
      {
        id: data.id || uuid(),
        name: data.name || null,
        description: data.description || null,
        units: data.units || null,
        standard_qty: data.standard_qty || data.price_standard_qty,
        cost_price: data.cost_price || data.price_standard_qty,
        pujari_selling_price:
          data.pujari_selling_price || data.price_standard_qty,
        customer_mrp: data.customer_mrp || data.price_standard_qty,
        karishye_provided: data.karishye_provided || "stocks",
        price_standard_qty: data.price_standard_qty || null,
        units_in_stock: data.units_in_stock || null,
        orderId: data.orderId || null,
        importHash: data.importHash || null,
        createdById: currentUser.id,
        updatedById: currentUser.id,
      },
      { transaction }
    );

    return samagri;
  }

  static async update(id, data, options) {
    const db = await getDbInstance();
    const transaction = (options && options.transaction) || undefined;

    const samagri = await db.samagri.findByPk(id, {
      transaction,
    });

    if (!samagri) {
      throw new Error('Samagri not found');
    }

    await samagri.update(
      {
        name: data.name || null,
        description: data.description || null,
        units: data.units || null,
        standard_qty: data.standard_qty || data.price_standard_qty,
        cost_price: data.cost_price || data.price_standard_qty,
        pujari_selling_price:
          data.pujari_selling_price || data.price_standard_qty,
        customer_mrp: data.customer_mrp || data.price_standard_qty,
        karishye_provided: data.karishye_provided || "stocks",
        price_standard_qty: data.price_standard_qty || null,
        units_in_stock: data.units_in_stock || null,
        orderId: data.orderId || null,
      },
      { transaction }
    );

    return samagri;
  }

  static async remove(id, options) {
    const db = await getDbInstance();
    const currentUser = (options && options.currentUser) || { id: null };
    const transaction = (options && options.transaction) || undefined;

    const samagri = await db.samagri.findByPk(id, options);

    if (!samagri) {
      throw new Error('Samagri not found');
    }

    await samagri.update(
      {
        deletedBy: currentUser.id,
      },
      {
        transaction,
      }
    );

    await samagri.destroy({
      transaction,
    });
    await Promise.all([
      db.puja_samagri_mappings.destroy(
        {
          where: {
            samagri_id: id,
          },
        },
        { transaction }
      ),

      db.booking_samagri_mappings.destroy(
        {
          where: {
            samagri_id: id,
          },
        },
        { transaction }
      ),
    ]);

    return samagri;
  }

  static async findBy(where, options) {
    const db = await getDbInstance();
    const transaction = (options && options.transaction) || undefined;

    const samagri = await db.samagri.findOne({ where }, { transaction });

    if (!samagri) {
      return samagri;
    }

    const output = samagri.get({ plain: true });

    return output;
  }

  static async findAll(filters = {}) {
    try {
      const dbInstance = await getDbInstance();

      const limit = Math.max(1, parseInt(filters.limit) || 20);
      const page = Math.max(1, parseInt(filters.page) || 1);
      const offset = (page - 1) * limit;

      // Remove pagination parameters from filters
      const where = { ...filters };
      delete where.limit;
      delete where.page;
      delete where.offset;

      // Remove empty string filters and undefined values
      Object.keys(where).forEach((key) => {
        if (where[key] === "" || where[key] === undefined) {
          delete where[key];
        }
      });

      // Only include valid columns in where clause
      const validColumns = [
        "id",
        "name",
        "description",
        "units",
        "standard_qty",
        "cost_price",
        "pujari_selling_price",
        "customer_mrp",
        "karishye_provided",
        "price_standard_qty",
        "units_in_stock",
        "orderId",
      ];

      const filteredWhere = {};
      Object.keys(where).forEach((key) => {
        if (validColumns.includes(key)) {
          filteredWhere[key] = where[key];
        }
      });
      const { rows, count } = await dbInstance.samagri.findAndCountAll({
        where: filteredWhere,
        limit,
        offset,
        raw: true,
      });

      return {
        rows,
        count,
        limit,
        offset,
        page,
      };
    } catch (error) {
      console.error("Error in Samagri.findAll:", error);
      throw error;
    }
  }

  static async findAllAutocomplete(query, limit) {
    let where = {};

    if (query) {
      where = {
        [Op.or]: [{ id: Utils.uuid(query) }, Utils.ilike("name", query)],
      };
    }
    const db = await getDbInstance();
    const records = await db.samagri.findAll({
      attributes: ["id", "name"], // Changed second "id" to "name"
      where,
      limit: limit ? Number(limit) : undefined,
      order: [["createdAt", "ASC"]], // Changed from "id" to "name"
    });

    return records.map((record) => ({
      id: record.id,
      label: record.name, // Changed from "id" to "name"
    }));
  }
};
