const initializeDatabase = require("../models");
const FileDBApi = require("./file");
const crypto = require("crypto");
const Utils = require("../utils");
const { Sequelize } = require("sequelize");
const bcrypt = require("bcrypt");
const config = require("../../api/config");

const Op = Sequelize.Op;
let db = null;

module.exports = class UsersDBApi {
  static async initialize() {
    if (!db) {
      db = await initializeDatabase();
    }
  }

  static async getDb() {
    if (!db) {
      await this.initialize();
    }
    return db;
  }

  static async create(data, options) {
    const db = await this.getDb();
    const currentUser = (options && options.currentUser) || { id: null };
    const transaction = (options && options.transaction) || undefined;

    const users = await db.users.create(
      {
        id: data.data.id || undefined,

        firstName: data.data.firstName || null,
        lastName: data.data.lastName || null,
        phoneNumber: data.data.phoneNumber || null,
        email: data.data.email || null,
        role: data.data.role || "user",

        disabled: data.data.disabled || false,

        password: data.data.password || null,
        emailVerified: data.data.emailVerified || true,

        emailVerificationToken: data.data.emailVerificationToken || null,
        emailVerificationTokenExpiresAt:
          data.data.emailVerificationTokenExpiresAt || null,
        passwordResetToken: data.data.passwordResetToken || null,
        passwordResetTokenExpiresAt:
          data.data.passwordResetTokenExpiresAt || null,
        provider: data.data.provider || null,
        type: data.data.type || null,
        email_verified_at: data.data.email_verified_at || null,
        first_time_login: data.data.first_time_login || null,
        forgot_password_token: data.data.forgot_password_token || null,
        remember_token: data.data.remember_token || null,
        forgot_password_token_timestamp:
          data.data.forgot_password_token_timestamp || null,
        is_active: data.data.is_active || null,
        importHash: data.data.importHash || null,
        createdById: currentUser.id,
        updatedById: currentUser.id,
      },
      { transaction }
    );

    await FileDBApi.replaceRelationFiles(
      {
        belongsTo: db.users.getTableName(),
        belongsToColumn: "avatar",
        belongsToId: users.id,
      },
      data.data.avatar,
      options
    );

    return users;
  }

  static async update(id, data, options) {
    const db = await this.getDb();
    const currentUser = (options && options.currentUser) || { id: null };
    const transaction = (options && options.transaction) || undefined;

    const users = await db.users.findByPk(id, {
      transaction,
    });

    if (data.password) {
      data.password = bcrypt.hashSync(data.password, config.bcrypt.saltRounds);
    } else {
      data.password = users.password;
    }

    await users.update(
      {
        firstName: data.firstName || null,
        lastName: data.lastName || null,
        phoneNumber: data.phoneNumber || null,
        email: data.email || null,
        role: data.role || "user",

        disabled: data.disabled || false,

        password: data.password || null,
        emailVerified: data.emailVerified || true,

        emailVerificationToken: data.emailVerificationToken || null,
        emailVerificationTokenExpiresAt:
          data.emailVerificationTokenExpiresAt || null,
        passwordResetToken: data.passwordResetToken || null,
        passwordResetTokenExpiresAt: data.passwordResetTokenExpiresAt || null,
        provider: data.provider || null,
        type: data.type || null,
        email_verified_at: data.email_verified_at || null,
        first_time_login: data.first_time_login || null,
        forgot_password_token: data.forgot_password_token || null,
        remember_token: data.remember_token || null,
        forgot_password_token_timestamp:
          data.forgot_password_token_timestamp || null,
        is_active: data.is_active || null,
        updatedById: currentUser.id,
      },
      { transaction }
    );

    await FileDBApi.replaceRelationFiles(
      {
        belongsTo: db.users.getTableName(),
        belongsToColumn: "avatar",
        belongsToId: users.id,
      },
      data.avatar,
      options
    );

    return users;
  }

  static async remove(id, options) {
    const db = await this.getDb();
    const currentUser = (options && options.currentUser) || { id: null };
    const transaction = (options && options.transaction) || undefined;

    const users = await db.users.findByPk(id, options);

    await users.update(
      {
        deletedBy: currentUser.id,
      },
      {
        transaction,
      }
    );

    await users.destroy({
      transaction,
    });

    return users;
  }

  static async findBy(where, options) {
    const db = await this.getDb();
    const transaction = (options && options.transaction) || undefined;

    const users = await db.users.findOne({ where }, { transaction });

    if (!users) {
      return users;
    }

    const output = users.get({ plain: true });

    output.avatar = await users.getAvatar({
      transaction,
    });

    return output;
  }

  static async findAll(filter, options) {
    const db = await this.getDb();
    var limit = filter.limit || 0;
    var offset = 0;
    const currentPage = +filter.page;

    offset = currentPage * limit;


    const transaction = (options && options.transaction) || undefined;
    let where = {};
    let include = [
      {
        model: db.file,
        as: "avatar",
      },
    ];

    if (filter) {
      if (filter.id) {
        where = {
          ...where,
          ["id"]: Utils.uuid(filter.id),
        };
      }

      if (filter.firstName) {
        where = {
          ...where,
          [Op.and]: Utils.ilike("firstName", filter.firstName),
        };
      }

      if (filter.lastName) {
        where = {
          ...where,
          [Op.and]: Utils.ilike("users", "lastName", filter.lastName),
        };
      }

      if (filter.phoneNumber) {
        where = {
          ...where,
          [Op.and]: Utils.ilike("users", "phoneNumber", filter.phoneNumber),
        };
      }

      if (filter.email) {
        where = {
          ...where,
          [Op.and]: Utils.ilike("users", "email", filter.email),
        };
      }

      if (filter.password) {
        where = {
          ...where,
          [Op.and]: Utils.ilike("users", "password", filter.password),
        };
      }

      if (filter.emailVerificationToken) {
        where = {
          ...where,
          [Op.and]: Utils.ilike(
            "users",
            "emailVerificationToken",
            filter.emailVerificationToken
          ),
        };
      }

      if (filter.passwordResetToken) {
        where = {
          ...where,
          [Op.and]: Utils.ilike(
            "users",
            "passwordResetToken",
            filter.passwordResetToken
          ),
        };
      }

      if (filter.provider) {
        where = {
          ...where,
          [Op.and]: Utils.ilike("users", "provider", filter.provider),
        };
      }

      if (filter.forgot_password_token) {
        where = {
          ...where,
          [Op.and]: Utils.ilike(
            "users",
            "forgot_password_token",
            filter.forgot_password_token
          ),
        };
      }

      if (filter.remember_token) {
        where = {
          ...where,
          [Op.and]: Utils.ilike(
            "users",
            "remember_token",
            filter.remember_token
          ),
        };
      }

      if (filter.emailVerificationTokenExpiresAtRange) {
        const [start, end] = filter.emailVerificationTokenExpiresAtRange;

        if (start !== undefined && start !== null && start !== "") {
          where = {
            ...where,
            emailVerificationTokenExpiresAt: {
              ...where.emailVerificationTokenExpiresAt,
              [Op.gte]: start,
            },
          };
        }

        if (end !== undefined && end !== null && end !== "") {
          where = {
            ...where,
            emailVerificationTokenExpiresAt: {
              ...where.emailVerificationTokenExpiresAt,
              [Op.lte]: end,
            },
          };
        }
      }

      if (filter.passwordResetTokenExpiresAtRange) {
        const [start, end] = filter.passwordResetTokenExpiresAtRange;

        if (start !== undefined && start !== null && start !== "") {
          where = {
            ...where,
            passwordResetTokenExpiresAt: {
              ...where.passwordResetTokenExpiresAt,
              [Op.gte]: start,
            },
          };
        }

        if (end !== undefined && end !== null && end !== "") {
          where = {
            ...where,
            passwordResetTokenExpiresAt: {
              ...where.passwordResetTokenExpiresAt,
              [Op.lte]: end,
            },
          };
        }
      }

      if (filter.email_verified_atRange) {
        const [start, end] = filter.email_verified_atRange;

        if (start !== undefined && start !== null && start !== "") {
          where = {
            ...where,
            email_verified_at: {
              ...where.email_verified_at,
              [Op.gte]: start,
            },
          };
        }

        if (end !== undefined && end !== null && end !== "") {
          where = {
            ...where,
            email_verified_at: {
              ...where.email_verified_at,
              [Op.lte]: end,
            },
          };
        }
      }

      if (filter.first_time_loginRange) {
        const [start, end] = filter.first_time_loginRange;

        if (start !== undefined && start !== null && start !== "") {
          where = {
            ...where,
            first_time_login: {
              ...where.first_time_login,
              [Op.gte]: start,
            },
          };
        }

        if (end !== undefined && end !== null && end !== "") {
          where = {
            ...where,
            first_time_login: {
              ...where.first_time_login,
              [Op.lte]: end,
            },
          };
        }
      }

      if (filter.forgot_password_token_timestampRange) {
        const [start, end] = filter.forgot_password_token_timestampRange;

        if (start !== undefined && start !== null && start !== "") {
          where = {
            ...where,
            forgot_password_token_timestamp: {
              ...where.forgot_password_token_timestamp,
              [Op.gte]: start,
            },
          };
        }

        if (end !== undefined && end !== null && end !== "") {
          where = {
            ...where,
            forgot_password_token_timestamp: {
              ...where.forgot_password_token_timestamp,
              [Op.lte]: end,
            },
          };
        }
      }

      if (filter.is_activeRange) {
        const [start, end] = filter.is_activeRange;

        if (start !== undefined && start !== null && start !== "") {
          where = {
            ...where,
            is_active: {
              ...where.is_active,
              [Op.gte]: start,
            },
          };
        }

        if (end !== undefined && end !== null && end !== "") {
          where = {
            ...where,
            is_active: {
              ...where.is_active,
              [Op.lte]: end,
            },
          };
        }
      }

      if (
        filter.active === true ||
        filter.active === "true" ||
        filter.active === false ||
        filter.active === "false"
      ) {
        where = {
          ...where,
          active: filter.active === true || filter.active === "true",
        };
      }

      if (filter.role) {
        where = {
          ...where,
          role: filter.role,
        };
      }

      if (filter.disabled) {
        where = {
          ...where,
          disabled: filter.disabled,
        };
      }

      if (filter.emailVerified) {
        where = {
          ...where,
          emailVerified: filter.emailVerified,
        };
      }

      if (filter.type) {
        where = {
          ...where,
          type: filter.type,
        };
      }

      if (filter.createdAtRange) {
        const [start, end] = filter.createdAtRange;

        if (start !== undefined && start !== null && start !== "") {
          where = {
            ...where,
            ["createdAt"]: {
              ...where.createdAt,
              [Op.gte]: start,
            },
          };
        }

        if (end !== undefined && end !== null && end !== "") {
          where = {
            ...where,
            ["createdAt"]: {
              ...where.createdAt,
              [Op.lte]: end,
            },
          };
        }
      }
    }

    let { rows, count } = await db.users.findAndCountAll({
      where,
      include,
      distinct: true,
      limit: limit ? Number(limit) : undefined,
      offset: offset ? Number(offset) : undefined,
      order:
        filter.field && filter.sort
          ? [[filter.field, filter.sort]]
          : [["updatedAt", "desc"]],
      transaction,
    });

    //    rows = await this._fillWithRelationsAndFilesForRows(
    //      rows,
    //      options,
    //    );

    return { rows, count };
  }

  static async findAllAutocomplete(query, limit) {
    const db = await this.getDb();
    let where = {};

    if (query) {
      where = {
        [Op.or]: [
          { ["id"]: Utils.uuid(query) },
          Utils.ilike("users", "id", query),
        ],
      };
    }

    const records = await db.users.findAll({
      attributes: ["id", "id"],
      where,
      limit: limit ? Number(limit) : undefined,
      orderBy: [["id", "ASC"]],
    });

    return records.map((record) => ({
      id: record.id,
      label: record.id,
    }));
  }

  static async createFromAuth(data, options) {
    const db = await this.getDb();
    const transaction = (options && options.transaction) || undefined;
    const users = await db.users.create(
      {
        email: data.email,
        firstName: data.firstName,
        authenticationUid: data.authenticationUid,
        password: data.password,
      },
      { transaction }
    );

    await users.update(
      {
        authenticationUid: users.id,
      },
      { transaction }
    );

    delete users.password;
    return users;
  }

  static async updatePassword(id, password, options) {
    const db = await this.getDb();
    const currentUser = (options && options.currentUser) || { id: null };

    const transaction = (options && options.transaction) || undefined;

    const users = await db.users.findByPk(id, {
      transaction,
    });

    await users.update(
      {
        password,
        authenticationUid: id,
        updatedById: currentUser.id,
      },
      { transaction }
    );

    return users;
  }

  static async generateEmailVerificationToken(email, options) {
    return this._generateToken(
      ["emailVerificationToken", "emailVerificationTokenExpiresAt"],
      email,
      options
    );
  }

  static async generatePasswordResetToken(email, options) {
    return this._generateToken(
      ["passwordResetToken", "passwordResetTokenExpiresAt"],
      email,
      options
    );
  }

  static async findByPasswordResetToken(token, options) {
    const db = await this.getDb();
    const transaction = (options && options.transaction) || undefined;

    return db.users.findOne(
      {
        where: {
          passwordResetToken: token,
          passwordResetTokenExpiresAt: {
            [db.Sequelize.Op.gt]: Date.now(),
          },
        },
      },
      { transaction }
    );
  }

  static async findByEmailVerificationToken(token, options) {
    const db = await this.getDb();
    const transaction = (options && options.transaction) || undefined;
    return db.users.findOne(
      {
        where: {
          emailVerificationToken: token,
          emailVerificationTokenExpiresAt: {
            [db.Sequelize.Op.gt]: Date.now(),
          },
        },
      },
      { transaction }
    );
  }

  static async markEmailVerified(id, options) {
    const db = await this.getDb();
    const currentUser = (options && options.currentUser) || { id: null };
    const transaction = (options && options.transaction) || undefined;

    const users = await db.users.findByPk(id, {
      transaction,
    });

    await users.update(
      {
        emailVerified: true,
        updatedById: currentUser.id,
      },
      { transaction }
    );

    return true;
  }

  static async _generateToken(keyNames, email, options) {
    const db = await this.getDb();
    const currentUser = (options && options.currentUser) || { id: null };
    const transaction = (options && options.transaction) || undefined;
    const users = await db.users.findOne(
      {
        where: { email: email.toLowerCase() },
      },
      {
        transaction,
      }
    );

    const token = crypto.randomBytes(20).toString("hex");
    const tokenExpiresAt = Date.now() + 360000;

    if (users) {
      await users.update(
        {
          [keyNames[0]]: token,
          [keyNames[1]]: tokenExpiresAt,
          updatedById: currentUser.id,
        },
        { transaction }
      );
    }

    return token;
  }
};
