"use strict";

const { v4: uuid } = require("uuid");
const dotenv = require("dotenv");
const fs = require("fs");
const path = require("path");
const { Sequelize, DataTypes } = require("sequelize");

const env = process.env.NODE_ENV || "development";
dotenv.config({ path: `./environments/.env.${env}` });

const basename = path.basename(__filename);
const config = {
  dialect: "mysql",
  host: process.env.DB_HOST,
  port: process.env.DB_PORT,
  username: process.env.DB_USERNAME,
  password: process.env.DB_PASSWORD,
  database: process.env.DB_DATABASE,
  logging: console.log,
  timestamps: false,
  seederStorage: "sequelize",
};

const maxRetries = 5;
const retryInterval = 5000; // 5 seconds

const connectWithRetry = async (config, attempt = 1) => {
  try {
    const sequelize = new Sequelize(
      config.database,
      config.username,
      config.password,
      {
        ...config,
        retry: {
          max: 3,
          timeout: 60000,
        },
        pool: {
          max: 5,
          min: 0,
          acquire: 30000,
          idle: 10000,
        },
      }
    );

    // Test the connection
    await sequelize.authenticate();
    console.log("Database connection established successfully.");
    return sequelize;
  } catch (error) {
    console.error(
      `Database connection attempt ${attempt} failed:`,
      error.message
    );

    if (attempt < maxRetries) {
      console.log(`Retrying in ${retryInterval / 1000} seconds...`);
      await new Promise((resolve) => setTimeout(resolve, retryInterval));
      return connectWithRetry(config, attempt + 1);
    }

    console.error("Max retries reached. Unable to connect to database.");
    throw error;
  }
};

let sequelize;
let db = {
  sequelize,
  Sequelize,
};

let dbInstance = null;

const initializeDatabase = async () => {
  if (dbInstance) {
    return dbInstance;
  }

  try {
    sequelize = await connectWithRetry(config);

    // Ensure we have a connection before proceeding
    await sequelize.authenticate();

    // Load models
    const modelFiles = fs
      .readdirSync(__dirname)
      .filter(
        (file) =>
          file.indexOf(".") !== 0 &&
          file !== basename &&
          file.slice(-3) === ".js"
      );

    for (const file of modelFiles) {
      try {
        const model = require(path.join(__dirname, file))(sequelize, DataTypes);
        if (model) {
          db[model.name] = model;
          console.log(`Loaded model: ${model.name}`);
        }
      } catch (error) {
        console.error(`Error loading model ${file}:`, error);
      }
    }

    // Initialize associations
    Object.keys(db).forEach((modelName) => {
      if (db[modelName] && db[modelName].associate) {
        db[modelName].associate(db);
      }
    });

    // Add UUID hooks
    [
      "puja_samagri_mappings",
      "puja_history",
      "samagri",
      "puja_model_procedure_mappings",
    ].forEach((modelName) => {
      if (db[modelName]) {
        db[modelName].beforeCreate((instance) => {
          instance.id = uuid();
        });
      }
    });

    db.sequelize = sequelize;
    db.Sequelize = Sequelize;

    dbInstance = db;
    return dbInstance;
  } catch (error) {
    console.error("Failed to initialize database:", error);
    throw error;
  }
};

// Add a method to check if DB is initialized
initializeDatabase.isInitialized = () => !!dbInstance;

// Add a method to get the current instance without initializing
initializeDatabase.getInstance = () => dbInstance;

module.exports = initializeDatabase;
