/* eslint-disable @typescript-eslint/no-unused-vars */
import { HttpService } from '@nestjs/axios';
import { Injectable, Logger } from '@nestjs/common';
import { messages } from 'src/common/messages';
import {
  BookingProcedureMappings,
  Bookings,
  BookingSamagriMappings,
  KarishyeUsers,
  Pujas,
} from 'src/entities';
import { PujaModels } from 'src/entities/puja-models';
import { Pujaris } from 'src/entities/pujaris';
import {
  BOOKING_CREATED_ADMIN,
  BOOKING_STATUS_ADMIN,
} from '../email-config/email-templates';
import { EmailService } from '../email-config/email.service';
import { WhatsappService } from '../whatsapp/whatsapp.service';
import { AppDataSource } from './../../common/db-utils';
import { APIFailedException } from './../../common/exceptions';
import {
  AssignedBookingQuery,
  BookingDetailsQuery,
  BookingSamagriMapping,
  CreateBookingPayload,
  EditBookingPayload,
  UpdateBookingPayload,
  UpdateBookingStatuses,
} from './models/createBooking.dto';
import { GetBookingsDto } from './models/getBookings.dto';
import { v4 as uuid } from 'uuid';
import { In } from 'typeorm';

@Injectable()
export class BookingsService {
  logger: Logger;

  constructor(
    private readonly httpService: HttpService,
    private whatsappService: WhatsappService,
    private emailService: EmailService
  ) {
    this.logger = new Logger();
  }

  async getBookings(query: GetBookingsDto): Promise<Bookings[] | null> {
    try {
      const date =
        query.startDate1 && query.startDate1.toString().split(' ')[0]
          ? query.startDate1.toString().split(' ')[0]
          : '';

      this.logger.debug('API Triggered');
      const repo = AppDataSource.getRepository(Bookings);

      let sqlQuery = repo
        .createQueryBuilder('booking')
        .innerJoinAndSelect(
          'booking.puja_model',
          'models',
          'models.id = booking.model'
        )
        .innerJoinAndSelect('models.puja', 'pujas', 'models.puja_id = pujas.id')
        .innerJoinAndSelect(
          'booking.user',
          'users',
          'users.id = booking.user_id'
        )
        .leftJoinAndSelect(
          'booking.bookingSamagriMappings',
          'samagriMappings',
          'samagriMappings.bookingId = booking.id'
        )
        .leftJoinAndSelect(
          'booking.bookingproceduremappings',
          'procedureMappings',
          'procedureMappings.bookingId = booking.id'
        )
        .leftJoinAndSelect(
          'samagriMappings.samagri',
          'samagri',
          'samagri.id = samagriMappings.samagriId'
        )
        .leftJoinAndSelect(
          'procedureMappings.procedure',
          'procedure',
          'procedure.id = procedureMappings.procedureId'
        )
        .where(`booking.pujari_id = ${query.pujariId}`)
        .andWhere(
          '(booking.bookingStatus = \'puja_confirmed\' or booking.bookingStatus = \'puja_started\')'
        );

      if (date.length > 0) {
        sqlQuery = sqlQuery.andWhere(
          `booking.start_date >= "${date} 00:00:00" and booking.start_date <= "${date} 23:59:59"`
        );
      }
      return await sqlQuery.getMany();
    } catch (error: any) {
      this.logger.error(error);
      throw new APIFailedException(error);
    }
  }

  async postBookings(
    bookingInfo: CreateBookingPayload
  ): Promise<Bookings[] | null> {
    try {
      this.logger.log(
        'Starting postBookings process with bookingInfo:',
        bookingInfo
      );

      await this.validateBookingInput(bookingInfo);

      const insertedBooking = await this.insertBooking(bookingInfo);

      await this.processBookingRelations(bookingInfo, insertedBooking);

      return insertedBooking;
    } catch (error: any) {
      this.logger.error('Error in postBookings', error);
      throw new APIFailedException(error);
    }
  }

  private async validateBookingInput(
    bookingInfo: CreateBookingPayload
  ): Promise<void> {
    // Validate user and pujari IDs
    const valid = await validateIds(
      bookingInfo.data.userId,
      bookingInfo.data.pujariId,
      bookingInfo.data.model
    );
    this.logger.log('Validation result:', valid);

    if (valid.status !== true) {
      throw new Error(valid);
    }

    // Additional validation for booking ID
    if (!(await validateBookingId(bookingInfo.data.id))) {
      throw new Error('Invalid booking ID');
    }

    // Ensure name and surname are not undefined
    bookingInfo.data.name = bookingInfo.data.name || '';
    bookingInfo.data.surname = bookingInfo.data.surname || '';

    // Set createdAt and updatedAt
    bookingInfo.data.createdAt = new Date();
    bookingInfo.data.updatedAt = new Date();
  }

  private async insertBooking(bookingInfo: CreateBookingPayload): Promise<any> {
    const repo = await AppDataSource.getRepository(Bookings).insert(
      bookingInfo.data
    );
    this.logger.log('Booking inserted with ID:', repo.raw.insertId);

    return repo;
  }

  private async processBookingRelations(
    bookingInfo: CreateBookingPayload,
    insertedBooking: any
  ): Promise<void> {
    const bookingId = insertedBooking.raw.insertId;

    // Prepare Samagri data
    const samagriData = this.prepareSamagriData(bookingId, bookingInfo.samagri);

    // Prepare Procedures data
    const proceduresMap = new Map<number, string>(
      bookingInfo.procedures.map((id, index) => [index, id.toString()])
    );
    const proceduresData = this.prepareProceduresData(bookingId, proceduresMap);

    // Insert Procedures and Samagri concurrently
    const [proceduresInsertion, samagriInsertion] = await Promise.all([
      AppDataSource.createQueryBuilder()
        .insert()
        .into(BookingProcedureMappings)
        .values(proceduresData)
        .execute(),
      AppDataSource.createQueryBuilder()
        .insert()
        .into(BookingSamagriMappings)
        .values(samagriData)
        .execute(),
    ]);

    this.logger.log('Procedures and Samagri inserted:', {
      proceduresInsertion,
      samagriInsertion,
    });

    await this.sendBookingNotifications(bookingId);
  }

  private prepareSamagriData(
    bookingId: number,
    samagri: BookingSamagriMapping[]
  ): any[] {
    return samagri.map((samagriData: BookingSamagriMapping) => ({
      bookingId,
      ...samagriData,
      id: uuid(),
    }));
  }

  private prepareProceduresData(
    bookingId: number,
    procedures: Map<number, string>
  ): any[] {
    return Array.from(procedures.entries()).map(([index, id]) => {
      // Ensure id is a number
      const procedureId = Number(id);
      if (isNaN(procedureId)) {
        throw new Error(`Invalid procedureId: ${id} is not a number`);
      }

      return {
        procedureId,
        bookingId,
        order: index,
      };
    });
  }

  private async sendBookingNotifications(bookingId: number): Promise<void> {
    try {
      // Fetch the booking data with relations
      const bookingRepo = await AppDataSource.getRepository(Bookings);
      const bookingData = await bookingRepo.find({
        where: { id: bookingId },
        relations: ['puja_model', 'puja_model.puja', 'user', 'pujaris'],
      });
      this.logger.log('Booking data fetched:', bookingData);

      // Fetch the Puja data
      const pujaData = await AppDataSource.getRepository(Pujas).findOne({
        where: { id: bookingData[0].puja_model.puja.id },
      });
      this.logger.log('Puja data fetched:', pujaData);

      // Send WhatsApp message to the admin
      const whatsappResult = this.whatsappService.sendWhatsappMessage(
        '918121007481', // Admin number
        'puja_created_admin',
        [
          { name: 'booking_id', value: bookingId },
          { name: 'username', value: bookingData[0].user.name },
        ]
      );

      this.logger.log('WhatsApp message result:', whatsappResult);

      // Send email to the admin
      this.emailService.sendMail('admin', {
        template: BOOKING_CREATED_ADMIN.testTemplate,
        mailProps: {
          to: [
            process.env.ADMIN_EMAIL_1 || '',
            process.env.ADMIN_EMAIL_2 || '',
            process.env.ADMIN_EMAIL_3 || '',
          ],
        },
        dynamicData: {
          bookingId: bookingData[0].id,
          mobile: `${bookingData[0].user.phoneNumber}`,
          username: bookingData[0].user.name,
          pujaName: bookingData[0].puja_model.puja.name,
        },
      });

      this.logger.log('Email sent to admin');
    } catch (error) {
      this.logger.error('Error in booking notification process', error);
      throw new APIFailedException(error);
    }
  }

  async getAssignedBookingstoPujari(
    data: AssignedBookingQuery
  ): Promise<Bookings[] | null> {
    try {
      this.logger.debug('Pujari assigned bookings API Triggered');
      const repo = AppDataSource.getRepository(Bookings);
      return await repo.find({
        where: {
          pujariId: Number(data.userId),
          bookingStatus: In([
            'pujari_assigned',
            'pujari_accepted',
            'plan_under_review',
            'plan_approved',
          ]),
        },
        relations: [
          'puja_model',
          'puja_model.puja',
          'user',
          'bookingSamagriMappings',
          'bookingSamagriMappings.samagri',
          'bookingproceduremappings.procedure',
        ],
      });
    } catch (error: any) {
      this.logger.error(error);
      throw new APIFailedException(error);
    }
  }

  async getAssignedBookings(
    query: AssignedBookingQuery
  ): Promise<Bookings[] | null> {
    try {
      this.logger.debug('Assigned bookings of customer API Triggered');
      const repo = AppDataSource.getRepository(Bookings);
      return await repo.find({
        where: { userId: query.userId },
        relations: [
          'puja_model',
          'puja_model.puja',
          'user',
          'pujaris',
          'bookingSamagriMappings.samagri',
          'bookingproceduremappings.procedure',
        ],
        order: { createdAt: 'DESC' },
      });
    } catch (error: any) {
      this.logger.error(error);
      throw new APIFailedException(error);
    }
  }

  async getPujaByBookingAndUserId(
    //API for getting specific puja by booking ID and user ID
    bookingId: number
  ): Promise<Bookings | null> {
    try {
      this.logger.debug(
        'Get specific puja by booking ID and user ID API Triggered'
      );
      const repo = AppDataSource.getRepository(Bookings);
      const booking = await repo.findOne({
        where: { id: bookingId }, //getting the booking by booking ID
        relations: [
          //relations to get the required data
          'puja_model',
          'puja_model.puja',
          'user',
          'pujaris',
          'bookingSamagriMappings.samagri',
          'bookingproceduremappings.procedure',
        ],
      });

      if (!booking) {
        throw new APIFailedException('Booking with userId is not found');
      }

      return booking;
    } catch (error: any) {
      this.logger.error(error);
      throw new APIFailedException(error);
    }
  }

  async updatePujaStatus(body: UpdateBookingPayload) {
    try {
      const repo = AppDataSource.getRepository(Bookings);

      const data = await repo
        .createQueryBuilder('booking')
        .update()
        .set({
          bookingStatus: body.status,
          startTime: body.start_time,
          endTime: body.end_time,
        })
        .where('id = :id', { id: body.id })
        .execute();

      if (data.affected === 0) {
        return { message: 'Booking is not updated' };
      }

      const bookingData = await repo.find({
        where: { id: body.id },
        relations: ['puja_model', 'puja_model.puja', 'user', 'pujaris'],
      });

      if (body.status === 'puja_started') {
        this.whatsappService.sendWhatsappMessage(
          `${bookingData[0].user.phoneNumber}`,
          'puja_status_update',
          [
            {
              name: 'status',
              value: 'puja started by pujari',
            },
          ]
        );

        this.emailService.sendMail('admin', {
          template: BOOKING_STATUS_ADMIN.testTemplate,
          mailProps: {
            to: [
              process.env.ADMIN_EMAIL_1 || '',
              process.env.ADMIN_EMAIL_2 || '',
              process.env.ADMIN_EMAIL_3 || '',
            ],
          },
          dynamicData: {
            status: 'puja started by pujari',
            bookingId: body.id,
          },
        });
      } else if (body.status === 'puja_ended') {
        this.whatsappService.sendWhatsappMessage(
          `${bookingData[0].user.phoneNumber}`,
          'puja_status_update',
          [
            {
              name: 'status',
              value: 'puja completed.',
            },
          ]
        );

        // this.whatsappService.sendWhatsappMessage(
        //   process.env.ADMIN_NUMBER ? process.env.ADMIN_NUMBER : "919381062567",
        //   "puja_status_update",
        //   [
        //     {
        //       name: "status",
        //       value: `puja completed.`,
        //     },
        //   ]
        // );
        // this.whatsappService.sendWhatsappMessage(
        //   process.env.ADMIN_NUMBER ? process.env.ADMIN_NUMBER : "919381062567",
        //   "puja_status_update",
        //   [
        //     {
        //       name: "status",
        //       value: `puja completed.`,
        //     },
        //   ]
        // );
        this.whatsappService.sendWhatsappMessage(
          process.env.ADMIN_NUMBER ? process.env.ADMIN_NUMBER : '918121007481',
          'puja_status_update',
          [
            {
              name: 'status',
              value: 'puja completed.',
            },
          ]
        );

        this.emailService.sendMail('admin', {
          template: BOOKING_STATUS_ADMIN.testTemplate,
          mailProps: {
            to: [
              process.env.ADMIN_EMAIL_1 || '',
              process.env.ADMIN_EMAIL_2 || '',
              process.env.ADMIN_EMAIL_3 || '',
            ],
          },
          dynamicData: {
            status: 'puja completed.',
            bookingId: body.id,
          },
        });
      }

      return await repo
        .createQueryBuilder()
        .select([
          'start_time as start_time',
          'end_time as end_time',
          'id as id',
        ])
        .where('id = :id', { id: body.id })
        .getRawOne();
    } catch (error: any) {
      this.logger.error(error);
      throw new APIFailedException(error);
    }
  }

  async updateBookingStatuses(bookingId: number, body: UpdateBookingStatuses) {
    try {
      const repo = AppDataSource.getRepository(Bookings);

      const updateObj = {
        bookingStatus: body.bookingStatus,
        paymentStatus: body.paymentStatus,
        samagriStatus: body.samagriStatus,
      };

      const data = await repo
        .createQueryBuilder('booking')
        .update()
        .set(updateObj)
        .where('id = :id', { id: bookingId })
        .execute();

      if (data.affected === 0) {
        return { message: 'Booking is not updated' };
      }

      const bookingData = await repo.find({
        where: { id: bookingId },
        relations: ['puja_model', 'puja_model.puja', 'user', 'pujaris'],
      });

      if (
        body.bookingStatus === 'pujari_accepted' ||
        body.bookingStatus === 'pujari_rejected'
      ) {
        this.handleStatusChange(body.bookingStatus, bookingData[0]);
      }

      return 'Booking Updated';
    } catch (error: any) {
      this.logger.error(error);
      throw new APIFailedException(error);
    }
  }

  private handleStatusChange(status: string, booking: any) {
    const statusMessage = `${booking.pujaris.name} ${status.replace(
      'pujari_',
      ''
    )} the booking`;
    const adminNumber = process.env.ADMIN_NUMBER || '918121007481';
    const adminEmails = [
      process.env.ADMIN_EMAIL_1 || '',
      process.env.ADMIN_EMAIL_2 || '',
      process.env.ADMIN_EMAIL_3 || '',
    ];

    this.whatsappService.sendWhatsappMessage(
      adminNumber,
      'puja_status_update',
      [
        {
          name: 'status',
          value: statusMessage,
        },
      ]
    );

    this.emailService.sendMail('admin', {
      template: BOOKING_STATUS_ADMIN.testTemplate,
      mailProps: {
        to: adminEmails,
      },
      dynamicData: {
        status: statusMessage,
        bookingId: booking.id,
      },
    });
  }

  removeGMTExtraTime(pickedDate: any) {
    if (pickedDate) {
      const dateTimeParts = pickedDate.split(/[-TZ:.]/);
      const [year, month, day, hours, minutes, seconds] =
        dateTimeParts.map(parseFloat);
      const utcDate = new Date(
        Date.UTC(year, month - 1, day, hours - 5, minutes - 30, seconds)
      );
      return utcDate;
    } else {
      return undefined;
    }
  }

  async editBooking(
    bookingId: number,
    bookingInfo: EditBookingPayload
  ): Promise<string | null> {
    const formattedMuhurtamDate: Date | undefined = bookingInfo.data.muhurtam;
    const formattedStartDate: any = bookingInfo.data.startDate;
    const formattedEndDate: any = bookingInfo.data.endDate?.toString();
    bookingInfo.data.muhurtam = formattedMuhurtamDate;
    bookingInfo.data.startDate = formattedStartDate;
    bookingInfo.data.endDate = formattedEndDate;
    bookingInfo.data.updatedAt = new Date();

    try {
      const bookingRepo = AppDataSource.getRepository(Bookings);
      const bookingProcedures = AppDataSource.getRepository(
        BookingProcedureMappings
      );
      const bookingSamagri = AppDataSource.getRepository(
        BookingSamagriMappings
      );

      const data = await bookingRepo
        .createQueryBuilder('booking')
        .update()
        .set(bookingInfo.data)
        .where('id = :id', { id: bookingId })
        .execute();

      // Delete existing booking procedures and samgris
      const deleteProceduresAndSamagris = await Promise.all([
        bookingProcedures.delete({ bookingId: bookingId }),
        bookingSamagri.delete({ bookingId: bookingId }),
      ]);
      const tempArr: any = bookingInfo.samagri.map(
        (samagriData: BookingSamagriMapping) => {
          return { id: uuid(), bookingId: bookingId, ...samagriData };
        }
      );

      const proceduresData = Array.from(bookingInfo.procedures.entries()).map(
        ([index, id]) => {
          return {
            procedureId: id,
            bookingId: bookingId,
            order: index,
          };
        }
      );

      const [proceduresInsertion, samagriInsertion] = await Promise.all([
        AppDataSource.createQueryBuilder()
          .insert()
          .into(BookingProcedureMappings)
          .values(
            // bookingInfo.procedures.map((id: number) => {
            //   return { procedureId: id, bookingId };
            // })
            proceduresData
          )
          .execute(),
        AppDataSource.createQueryBuilder()
          .insert()
          .into(BookingSamagriMappings)
          .values(tempArr)
          .execute(),
      ]);

      const repo = AppDataSource.getRepository(Bookings);

      const bookingData = await repo.find({
        where: { id: bookingId },
        relations: ['puja_model', 'puja_model.puja', 'user', 'pujaris'],
      });

      const url = `${process.env.WATI_URL}/api/v1/sendTemplateMessage?whatsappNumber=${bookingData[0].user.phoneNumber}`;
      const options = {
        headers: {
          'content-type': 'text/json',
          Authorization: `${process.env.WATI_API_KEY}`,
        },
        body: {
          broadcast_name: 'New-Booking',
          template_name: 'puja_details_edit',
          parameters: [
            { name: 'booking_id', value: bookingId },
            { name: 'puja_name', value: bookingData[0].puja_model.puja.name },
            { name: 'pujari_name', value: bookingData[0].pujaris.name },
            { name: 'customer_name', value: bookingData[0].user.name },
            { name: 'price', value: bookingData[0].bookingSellingPrice },
          ],
        },
      };

      let result;

      // lastValueFrom(
      //   this.httpService
      //     .post(url, options.body, { headers: options.headers })
      //     .pipe(map((res) => res.data))
      // ).then((result) => (result = result));

      // this.emailService.sendMail({
      //   template: BOOKING_EDITED.testTemplate,
      //   mailProps: {
      //     to: ["phanindra@divami.com", `${bookingData[0].user.emailId}`],
      //   },
      //   dynamicData: {
      //     pujariName: bookingData[0].pujaris.name,
      //     bookingId: bookingId,
      //     price: bookingData[0].bookingSellingPrice,
      //     puja: bookingData[0].puja_model.puja.name,
      //     customer: bookingData[0].user.name,
      //   },
      // });

      return 'Booking data updated';
    } catch (error: any) {
      this.logger.error(error);
      throw new APIFailedException(error);
    }
  }

  async getBookingDetails(query: BookingDetailsQuery) {
    try {
      const bookingRepo = AppDataSource.getRepository(Bookings);
      if (query.id === undefined) {
        throw new Error('Booking Id is missing');
      }

      const data = await bookingRepo.find({
        select: {
          bookingSamagriMappings: {
            id: true,
            isKarishyeProvided: true,
            qty: true,
            samagriType: true,
            isProvidedBy: true,
            samagri: {
              id: true,
              name: true,
              description: true,
              costPrice: true,
              units: true,
              standardQty: true,
              customerMrp: true,
              orderId: true,
            },
          },
        },
        where: { id: Number(query.id) },
        relations: [
          'puja_model',
          'puja_model.puja',
          'puja_model.puja.pujaSamagriMappings',
          'user',
          'pujaris',
          'bookingSamagriMappings.samagri',
          'bookingproceduremappings.procedure',
        ],
      });

      return data;
    } catch (error) {
      return new APIFailedException(error);
    }
  }
}

export const validateIds = async (userId: any, pujariId: any, modelId: any) => {
  try {
    const userInfo = await AppDataSource.getRepository(KarishyeUsers).find({
      where: { id: userId },
    });
    const pujariInfo = await AppDataSource.getRepository(Pujaris).find({
      where: { id: pujariId },
    });
    const modelInfo = await AppDataSource.getRepository(PujaModels).find({
      where: { id: modelId },
    });
    if (!userInfo || userInfo.length < 1) {
      const message = messages.USER_ID_MISMATCH({ userId: userId });
      return message;
    } else if (!pujariInfo || pujariInfo.length < 1) {
      const message = messages.PUJARI_ID_MISMATCH({ pujariId: pujariId });
      return message;
    } else if (!modelInfo || modelInfo.length < 1) {
      const message = messages.MODEL_ID_MISMATCH({ modelId: modelId });
      return message;
    }

    return { status: true, userInfo };
  } catch (err) {
    return err;
  }
};

export const validateBookingId = async (id: any) => {
  try {
    const bookingInfo = await AppDataSource.getRepository(Bookings).find({
      where: { id: id },
    });
    if (!bookingInfo || bookingInfo.length < 1) {
      const message = messages.BOOKING_ID_MISMATCH({ id: id });
      return message;
    }

    return { status: true, bookingInfo };
  } catch (err) {
    return err;
  }
};