import {
  Controller,
  Get,
  Logger,
  Param,
  ParseIntPipe,
  Query,
} from '@nestjs/common';
import { ApiOperation } from '@nestjs/swagger';
import { sendResponse } from 'src/common/common-utils';
import { APIFailedException } from 'src/common/exceptions';
import { KarishyeUsers, Pujas } from 'src/entities';
import { PujaModels } from 'src/entities/puja-models';
import { Samagri } from 'src/entities/samagri';
import {
  GetPujaModelsDto,
  GetPujasPayload,
  PujaHistoryListData,
} from './models/pujas.dto';
import { PujariService } from './pujari.service';

interface PujaModelsResponse {
  status: string;
  data: PujaModels[];
}

interface PujaHistoryResponse {
  status: string;
  data: PujaHistoryListData[];
}

interface CustomerResponse {
  status: string;
  data: KarishyeUsers[];
}

interface PujasResponse {
  status: string;
  data: Pujas[];
}
@Controller('/')
export class PujariController {
  logger: Logger;

  constructor(private readonly pujariService: PujariService) {
    this.logger = new Logger();
  }

  @Get('pujas-models')
  @ApiOperation({ summary: 'Get the list of Puja models' })
  async pujaModels(
    @Query() query: GetPujaModelsDto
  ): Promise<PujaModelsResponse> {
    try {
      const result = await this.pujariService.getPujaModels(query);

      if (!result) {
        throw new APIFailedException('No puja models found');
      }

      return {
        status: 'success',
        data: result,
      };
    } catch (error) {
      this.logger.error('Failed to get puja models:', error);
      throw new APIFailedException(error);
    }
  }

  // @Get("pujas")
  // @ApiOperation({ summary: "Get the list of Pujas along with history" })
  // async pujas(
  //   @Query() query: GetPujasDto
  // ): Promise<{ status: String; data: Pujas[] } | null> {
  //   const re = await this.pujariService.getPujas(query);
  //   this.logger.debug("API Triggered", re);

  //   return sendResponse(re);
  // }

  @Get('puja-samagri/:pujaId')
  @ApiOperation({ summary: 'Get the list of Puja samagris' })
  async pujaSamagri(
    @Param('pujaId', new ParseIntPipe()) id: number
  ): Promise<{ status: string; data: Samagri[] }> {
    try {
      const result = await this.pujariService.getPujaSamagri(id);

      if (!result) {
        throw new APIFailedException('No samagri found for this puja');
      }

      return {
        status: 'success',
        data: result as Samagri[],
      };
    } catch (error) {
      this.logger.error('Failed to get puja samagri:', error);
      throw new APIFailedException(error);
    }
  }

  @Get('puja-history')
  @ApiOperation({ summary: 'Get the history of all the pujas' })
  async pujaHistory(): Promise<PujaHistoryResponse> {
    try {
      const result = await this.pujariService.getPujaHistory();

      if (!result) {
        throw new APIFailedException('No puja history found');
      }

      return {
        status: 'success',
        data: result as unknown as PujaHistoryListData[],
      };
    } catch (error) {
      this.logger.error('Failed to get puja history:', error);
      throw new APIFailedException(error);
    }
  }

  @Get('existing-customers/:pujariId')
  @ApiOperation({ summary: 'Get the list of existing customers for a pujari' })
  async existingCustomers(
    @Param('pujariId', new ParseIntPipe()) id: number
  ): Promise<CustomerResponse> {
    try {
      const result = await this.pujariService.getExistingCustomers(id);

      if (!result) {
        throw new APIFailedException('No customers found');
      }

      return {
        status: 'success',
        data: result as KarishyeUsers[],
      };
    } catch (error) {
      this.logger.error('Failed to get existing customers:', error);
      throw new APIFailedException(error);
    }
  }

  // @Get('puja-bookings')
  // async bookings(): Promise<Bookings[] | null> {
  //  return await this.pujariService.getBookings()
  // }

  @Get('pujas')
  @ApiOperation({ summary: 'Get the list of Pujas along with history' })
  async pujas(@Query() body: GetPujasPayload): Promise<PujasResponse> {
    try {
      this.logger.debug('API Triggered - Get pujas');
      const result = await this.pujariService.getPujas(body);

      if (!result) {
        throw new APIFailedException('No pujas found');
      }

      return {
        status: 'success',
        data: result,
      };
    } catch (error) {
      this.logger.error('Failed to get pujas:', error);
      throw new APIFailedException(error);
    }
  }

  @Get('all-pujas')
  @ApiOperation({
    summary: 'Get the list of Pujas in category - sub category wise',
  })
  async getPujaListCategorywise(): Promise<PujasResponse> {
    try {
      this.logger.debug('API Triggered - Get all pujas');
      const result = await this.pujariService.getAllPujas();

      if (!result) {
        throw new APIFailedException('No pujas found');
      }

      return {
        status: 'success',
        data: result as unknown as Pujas[],
      };
    } catch (error) {
      this.logger.error('Failed to get all pujas:', error);
      throw new APIFailedException(error);
    }
  }

  @Get('metadata')
  @ApiOperation({
    summary:
      'Get the meta data for the website like countries, states, cities, languages, nakshatram',
  })
  async getMetadata() {
    try {
      this.logger.debug(
        'API Triggered - Get the meta data for the website like countries, states, cities, languages, nakshatram'
      );
      return sendResponse(await this.pujariService.getMetadata());
    } catch (error: any) {
      this.logger.error(error);
      throw new APIFailedException(error);
    }
  }
}
