import { Body, Controller, Get, Post, Query, Logger } from '@nestjs/common';
import {
  ApiCreatedResponse,
  ApiOkResponse,
  ApiOperation,
} from '@nestjs/swagger';
import { BookingTransactions } from 'src/entities';
import { RazorpayService } from './razorpay.service';
import { APIFailedException } from 'src/common/exceptions';
import {
  CreateBookingTransactionDto,
  TransactionQuery,
} from './models/transaction.dto';

interface TransactionResponse {
  status: string;
  data: BookingTransactions[];
}
@Controller()
export class RazorpayController {
  logger: Logger;
  constructor(private readonly razorpayService: RazorpayService) {
    this.logger = new Logger();
  }
  @Post('transaction')
  @ApiOperation({ summary: 'Create transaction' })
  @ApiCreatedResponse({ type: CreateBookingTransactionDto })
  async createTransaction(
    @Body() body: CreateBookingTransactionDto
  ): Promise<TransactionResponse> {
    try {
      const result = await this.razorpayService.postTransaction(body);

      if (!result) {
        throw new APIFailedException('Failed to create transaction');
      }

      return {
        status: 'success',
        data: result as unknown as BookingTransactions[],
      };
    } catch (error) {
      this.logger.error('Failed to create transaction:', error);
      throw new APIFailedException(error);
    }
  }

  @Get('booking-transaction')
  @ApiOperation({ summary: 'Get the list of transactions for a booking' })
  @ApiOkResponse({ type: CreateBookingTransactionDto, isArray: true })
  async getTransactions(
    @Query() query: TransactionQuery
  ): Promise<TransactionResponse> {
    try {
      const result = await this.razorpayService.getTransactions(query);

      if (!result) {
        throw new APIFailedException('No transactions found');
      }

      return {
        status: 'success',
        data: result,
      };
    } catch (error) {
      this.logger.error('Failed to get transactions:', error);
      throw new APIFailedException(error);
    }
  }
}
