import { Injectable, Logger } from '@nestjs/common';
import Razorpay from 'razorpay';
import { AppDataSource } from '../common/db-utils';
import { APIFailedException } from '../common/exceptions';
import {
  CreateBookingTransactionDto,
  TransactionQuery,
} from './models/transaction.dto';
import { Bookings, BookingTransactions } from 'src/entities';
@Injectable()
export class RazorpayService {
  private readonly logger = new Logger(RazorpayService.name);
  private razorpayInstance: Razorpay;

  constructor() {
    const keyId = process.env.RAZORPAY_KEY_ID;
    const keySecret = process.env.RAZORPAY_KEY_SECRET;

    if (!keyId || !keySecret) {
      throw new Error('Razorpay key ID and secret must be defined');
    }

    this.razorpayInstance = new Razorpay({
      key_id: keyId,
      key_secret: keySecret,
    });
  }

  async createOrder(options: any): Promise<any> {
    return this.razorpayInstance.orders.create(options);
  }
  async initiatePayment(options: any): Promise<any> {
    this.logger.log(options);
    try {
      const order = this.razorpayInstance.orders.create(options);
      this.logger.log(order);
      return order;
    } catch (error) {
      console.error('Error initiating payment:', error.message);
      throw new Error(`Error initiating payment: ${error.message}`);
    }
  }

  async postTransaction(
    transactionInfo: CreateBookingTransactionDto
  ): Promise<BookingTransactions | null> {
    try {
      const bookingRepo = AppDataSource.getRepository(Bookings);
      const booking = await bookingRepo.findOne({
        where: { id: transactionInfo.booking_id },
      });

      if (!booking) {
        throw new APIFailedException('Invalid booking ID');
      }

      const transactionRepo = AppDataSource.getRepository(BookingTransactions);
      const transaction = transactionRepo.create(transactionInfo);
      await transactionRepo.save(transaction);

      return transaction;
    } catch (error: any) {
      this.logger.error(error);
      throw new APIFailedException(error);
    }
  }

  async getTransactions(
    query: TransactionQuery
  ): Promise<BookingTransactions[] | null> {
    try {
      const transactionRepo = AppDataSource.getRepository(BookingTransactions);
      const transactions = await transactionRepo.find({
        where: { booking_id: query.booking_id },
        select: [
          'transaction_id',
          'razorpay_id',
          'paidAmount',
          'totalAmount',
          'payment_status',
          'payment_type',
        ],
      });
      return transactions;
    } catch (error) {
      this.logger.error(
        `Error fetching transactions for booking ID ${query.booking_id}`,
        error
      );
      throw error;
    }
  }
}
